#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from datetime import datetime

from airflow import DAG
from airflow.providers.amazon.aws.operators.rds import (
    RdsCancelExportTaskOperator,
    RdsCopyDbSnapshotOperator,
    RdsCreateDbSnapshotOperator,
    RdsCreateEventSubscriptionOperator,
    RdsDeleteDbSnapshotOperator,
    RdsDeleteEventSubscriptionOperator,
    RdsStartExportTaskOperator,
)
from airflow.providers.amazon.aws.sensors.rds import RdsExportTaskExistenceSensor, RdsSnapshotExistenceSensor

# [START rds_snapshots_howto_guide]
with DAG(
    dag_id='rds_snapshots', start_date=datetime(2021, 1, 1), schedule_interval=None, catchup=False
) as dag:
    # [START howto_guide_rds_create_snapshot]
    create_snapshot = RdsCreateDbSnapshotOperator(
        task_id='create_snapshot',
        db_type='instance',
        db_identifier='auth-db',
        db_snapshot_identifier='auth-db-snap',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_create_snapshot]

    # [START howto_guide_rds_copy_snapshot]
    copy_snapshot = RdsCopyDbSnapshotOperator(
        task_id='copy_snapshot',
        db_type='instance',
        target_db_snapshot_identifier='auth-db-snap-backup',
        source_db_snapshot_identifier='auth-db-snap',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_copy_snapshot]

    # [START howto_guide_rds_delete_snapshot]
    delete_snapshot = RdsDeleteDbSnapshotOperator(
        task_id='delete_snapshot',
        db_type='instance',
        db_snapshot_identifier='auth-db-snap-backup',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_delete_snapshot]

    create_snapshot >> copy_snapshot >> delete_snapshot
# [END rds_snapshots_howto_guide]

# [START rds_exports_howto_guide]
with DAG(dag_id='rds_exports', start_date=datetime(2021, 1, 1), schedule_interval=None, catchup=False) as dag:
    # [START howto_guide_rds_start_export]
    start_export = RdsStartExportTaskOperator(
        task_id='start_export',
        export_task_identifier='export-auth-db-snap-{{ ds }}',
        source_arn='arn:aws:rds:<region>:<account number>:snapshot:auth-db-snap',
        s3_bucket_name='my_s3_bucket',
        s3_prefix='some/prefix',
        iam_role_arn='arn:aws:iam:<region>:<account number>:role/MyRole',
        kms_key_id='arn:aws:kms:<region>:<account number>:key/*****-****-****-****-********',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_start_export]

    # [START howto_guide_rds_cancel_export]
    cancel_export = RdsCancelExportTaskOperator(
        task_id='cancel_export',
        export_task_identifier='export-auth-db-snap-{{ ds }}',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_cancel_export]

    start_export >> cancel_export
# [END rds_exports_howto_guide]

# [START rds_events_howto_guide]
with DAG(dag_id='rds_events', start_date=datetime(2021, 1, 1), schedule_interval=None, catchup=False) as dag:
    # [START howto_guide_rds_create_subscription]
    create_subscription = RdsCreateEventSubscriptionOperator(
        task_id='create_subscription',
        subscription_name='my-topic-subscription',
        sns_topic_arn='arn:aws:sns:<region>:<account number>:MyTopic',
        source_type='db-instance',
        source_ids=['auth-db'],
        event_categories=['Availability', 'Backup'],
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_create_subscription]

    # [START howto_guide_rds_delete_subscription]
    delete_subscription = RdsDeleteEventSubscriptionOperator(
        task_id='delete_subscription',
        subscription_name='my-topic-subscription',
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_delete_subscription]

    create_subscription >> delete_subscription
# [END rds_events_howto_guide]

# [START rds_sensors_howto_guide]
with DAG(dag_id='rds_events', start_date=datetime(2021, 1, 1), schedule_interval=None, catchup=False) as dag:
    # [START howto_guide_rds_snapshot_sensor]
    snapshot_sensor = RdsSnapshotExistenceSensor(
        task_id='snapshot_sensor',
        db_type='instance',
        db_snapshot_identifier='auth-db-snap-{{ ds }}',
        target_statuses=['available'],
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_snapshot_sensor]

    # [START howto_guide_rds_export_sensor]
    export_sensor = RdsExportTaskExistenceSensor(
        task_id='export_sensor',
        export_task_identifier='export-auth-db-snap-{{ ds }}',
        target_statuses=['starting', 'in_progress', 'complete', 'canceling', 'canceled'],
        aws_conn_id='aws_default',
        hook_params={'region_name': 'us-east-1'},
    )
    # [END howto_guide_rds_export_sensor]
# [END rds_sensors_howto_guide]



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from datetime import datetime
from os import environ

from airflow import DAG
from airflow.providers.amazon.aws.operators.sagemaker import (
    SageMakerDeleteModelOperator,
    SageMakerModelOperator,
    SageMakerProcessingOperator,
    SageMakerTrainingOperator,
    SageMakerTransformOperator,
)

MODEL_NAME = "sample_model"
TRAINING_JOB_NAME = "sample_training"
IMAGE_URI = environ.get("ECR_IMAGE_URI", "123456789012.dkr.ecr.us-east-1.amazonaws.com/repo_name")
S3_BUCKET = environ.get("BUCKET_NAME", "test-airflow-12345")
ROLE = environ.get("SAGEMAKER_ROLE_ARN", "arn:aws:iam::123456789012:role/role_name")

SAGEMAKER_PROCESSING_JOB_CONFIG = {
    "ProcessingJobName": "sample_processing_job",
    "ProcessingInputs": [
        {
            "InputName": "input",
            "AppManaged": False,
            "S3Input": {
                "S3Uri": f"s3://{S3_BUCKET}/preprocessing/input/",
                "LocalPath": "/opt/ml/processing/input/",
                "S3DataType": "S3Prefix",
                "S3InputMode": "File",
                "S3DataDistributionType": "FullyReplicated",
                "S3CompressionType": "None",
            },
        },
    ],
    "ProcessingOutputConfig": {
        "Outputs": [
            {
                "OutputName": "output",
                "S3Output": {
                    "S3Uri": f"s3://{S3_BUCKET}/preprocessing/output/",
                    "LocalPath": "/opt/ml/processing/output/",
                    "S3UploadMode": "EndOfJob",
                },
                "AppManaged": False,
            }
        ]
    },
    "ProcessingResources": {
        "ClusterConfig": {
            "InstanceCount": 1,
            "InstanceType": "ml.m5.large",
            "VolumeSizeInGB": 5,
        }
    },
    "StoppingCondition": {"MaxRuntimeInSeconds": 3600},
    "AppSpecification": {
        "ImageUri": f"{IMAGE_URI}",
        "ContainerEntrypoint": ["python3", "./preprocessing.py"],
    },
    "RoleArn": f"{ROLE}",
}

SAGEMAKER_TRAINING_JOB_CONFIG = {
    "AlgorithmSpecification": {
        "TrainingImage": f"{IMAGE_URI}",
        "TrainingInputMode": "File",
    },
    "InputDataConfig": [
        {
            "ChannelName": "config",
            "DataSource": {
                "S3DataSource": {
                    "S3DataType": "S3Prefix",
                    "S3Uri": f"s3://{S3_BUCKET}/config/",
                    "S3DataDistributionType": "FullyReplicated",
                }
            },
            "CompressionType": "None",
            "RecordWrapperType": "None",
        },
    ],
    "OutputDataConfig": {
        "KmsKeyId": "",
        "S3OutputPath": f"s3://{S3_BUCKET}/training/",
    },
    "ResourceConfig": {
        "InstanceType": "ml.m5.large",
        "InstanceCount": 1,
        "VolumeSizeInGB": 5,
    },
    "StoppingCondition": {"MaxRuntimeInSeconds": 6000},
    "RoleArn": f"{ROLE}",
    "EnableNetworkIsolation": False,
    "EnableInterContainerTrafficEncryption": False,
    "EnableManagedSpotTraining": False,
    "TrainingJobName": TRAINING_JOB_NAME,
}

SAGEMAKER_CREATE_MODEL_CONFIG = {
    "ModelName": MODEL_NAME,
    "Containers": [
        {
            "Image": f"{IMAGE_URI}",
            "Mode": "SingleModel",
            "ModelDataUrl": f"s3://{S3_BUCKET}/training/{TRAINING_JOB_NAME}/output/model.tar.gz",
        }
    ],
    "ExecutionRoleArn": f"{ROLE}",
    "EnableNetworkIsolation": False,
}

SAGEMAKER_INFERENCE_CONFIG = {
    "TransformJobName": "sample_transform_job",
    "ModelName": MODEL_NAME,
    "TransformInput": {
        "DataSource": {
            "S3DataSource": {
                "S3DataType": "S3Prefix",
                "S3Uri": f"s3://{S3_BUCKET}/config/config_date.yml",
            }
        },
        "ContentType": "application/x-yaml",
        "CompressionType": "None",
        "SplitType": "None",
    },
    "TransformOutput": {"S3OutputPath": f"s3://{S3_BUCKET}/inferencing/output/"},
    "TransformResources": {"InstanceType": "ml.m5.large", "InstanceCount": 1},
}

# [START howto_operator_sagemaker]
with DAG(
    "sample_sagemaker_dag",
    schedule_interval=None,
    start_date=datetime(2022, 2, 21),
    catchup=False,
) as dag:
    sagemaker_processing_task = SageMakerProcessingOperator(
        config=SAGEMAKER_PROCESSING_JOB_CONFIG,
        aws_conn_id="aws_default",
        task_id="sagemaker_preprocessing_task",
    )

    training_task = SageMakerTrainingOperator(
        config=SAGEMAKER_TRAINING_JOB_CONFIG, aws_conn_id="aws_default", task_id="sagemaker_training_task"
    )

    model_create_task = SageMakerModelOperator(
        config=SAGEMAKER_CREATE_MODEL_CONFIG, aws_conn_id="aws_default", task_id="sagemaker_create_model_task"
    )

    inference_task = SageMakerTransformOperator(
        config=SAGEMAKER_INFERENCE_CONFIG, aws_conn_id="aws_default", task_id="sagemaker_inference_task"
    )

    model_delete_task = SageMakerDeleteModelOperator(
        task_id="sagemaker_delete_model_task", config={'ModelName': MODEL_NAME}, aws_conn_id="aws_default"
    )

    sagemaker_processing_task >> training_task >> model_create_task >> inference_task >> model_delete_task
    # [END howto_operator_sagemaker]
